'use strict'

exports.__esModule = true
exports.execute = void 0

var _graphql = require('graphql')

var _checkGraphQLSchema = require('./checkGraphQLSchema.js')

var _checkGraphQLValidationRules = require('./checkGraphQLValidationRules.js')

var _checkOptions = require('./checkOptions.js')

var _createHttpError = require('./createHttpError.js')

var _isEnumerableObject = require('./isEnumerableObject.js')

const ALLOWED_EXECUTE_OPTIONS = [
  'schema',
  'validationRules',
  'rootValue',
  'contextValue',
  'fieldResolver',
  'override'
]

const execute = options => {
  if (typeof options === 'undefined')
    throw (0, _createHttpError.createHttpError)(
      'GraphQL execute middleware options missing.'
    )
  ;(0, _checkOptions.checkOptions)(
    options,
    ALLOWED_EXECUTE_OPTIONS,
    'GraphQL execute middleware'
  )
  if (typeof options.schema !== 'undefined')
    (0, _checkGraphQLSchema.checkGraphQLSchema)(
      options.schema,
      'GraphQL execute middleware `schema` option'
    )
  if (typeof options.validationRules !== 'undefined')
    (0, _checkGraphQLValidationRules.checkGraphQLValidationRules)(
      options.validationRules,
      'GraphQL execute middleware `validationRules` option'
    )
  const { override, ...staticOptions } = options
  if (typeof override !== 'undefined' && typeof override !== 'function')
    throw (0, _createHttpError.createHttpError)(
      'GraphQL execute middleware `override` option must be a function.'
    )
  return async (ctx, next) => {
    if (typeof ctx.request.body === 'undefined')
      throw (0, _createHttpError.createHttpError)('Request body missing.')
    if (!(0, _isEnumerableObject.isEnumerableObject)(ctx.request.body))
      throw (0, _createHttpError.createHttpError)(
        400,
        'Request body must be a JSON object.'
      )
    if (!('query' in ctx.request.body))
      throw (0, _createHttpError.createHttpError)(
        400,
        'GraphQL operation field `query` missing.'
      )
    let document

    try {
      document = (0, _graphql.parse)(
        new _graphql.Source(ctx.request.body.query)
      )
    } catch (error) {
      throw (0, _createHttpError.createHttpError)(
        400,
        `GraphQL query syntax error: ${error.message}`
      )
    }

    let overrideOptions

    if (override) {
      overrideOptions = await override(ctx)
      ;(0, _checkOptions.checkOptions)(
        overrideOptions,
        ALLOWED_EXECUTE_OPTIONS.filter(option => option !== 'override'),
        'GraphQL execute middleware `override` option resolved'
      )
      if (typeof overrideOptions.schema !== 'undefined')
        (0, _checkGraphQLSchema.checkGraphQLSchema)(
          overrideOptions.schema,
          'GraphQL execute middleware `override` option resolved `schema` option'
        )
      if (typeof overrideOptions.validationRules !== 'undefined')
        (0, _checkGraphQLValidationRules.checkGraphQLValidationRules)(
          overrideOptions.validationRules,
          'GraphQL execute middleware `override` option resolved `validationRules` option'
        )
    }

    const requestOptions = {
      validationRules: [],
      ...staticOptions,
      ...overrideOptions
    }
    if (typeof requestOptions.schema === 'undefined')
      throw (0, _createHttpError.createHttpError)(
        'GraphQL execute middleware requires a GraphQL schema.'
      )
    const queryValidationErrors = (0, _graphql.validate)(
      requestOptions.schema,
      document,
      [..._graphql.specifiedRules, ...requestOptions.validationRules]
    )
    if (queryValidationErrors.length)
      throw (0, _createHttpError.createHttpError)(
        400,
        'GraphQL query validation errors.',
        {
          graphqlErrors: queryValidationErrors
        }
      )
    let result

    try {
      result = await (0, _graphql.execute)({
        schema: requestOptions.schema,
        rootValue: requestOptions.rootValue,
        contextValue: requestOptions.contextValue,
        fieldResolver: requestOptions.fieldResolver,
        document,
        variableValues: ctx.request.body.variables,
        operationName: ctx.request.body.operationName
      })
    } catch (error) {
      throw (0, _createHttpError.createHttpError)(
        400,
        `GraphQL operation field invalid: ${error.message}`
      )
    }

    if (result.data)
      ctx.response.body = {
        data: result.data
      }
    if (result.errors)
      throw (0, _createHttpError.createHttpError)(200, 'GraphQL errors.', {
        graphqlErrors: result.errors
      })
    ctx.response.status = 200
    await next()
  }
}

exports.execute = execute
